@@ -25,6 +25,7 @@
 
 #include "basic.h"
 #include "ptr.h"
+#include "optional.hpp"
 
 #include <iosfwd>
 #include <iterator>
@@ -74,9 +75,13 @@ class const_postorder_iterator;
 class symbol;
 struct symbolhasher;
 using symbolset = std::unordered_set<symbol,symbolhasher>;
+class wildcard;
+struct wildhasher;
+using wildset = std::unordered_set<wildcard,wildhasher>;
 using expairvec = std::vector<std::pair<ex,ex>>;
 using ocvector = std::vector<numeric>;
 using power_ocvector_map = std::map<ex, ocvector, GiNaC::ex_is_less>;
+using opt_ex = nonstd::optional<ex>;
 
 /** Lightweight wrapper for GiNaC's symbolic objects.  It holds a pointer to
  *  the other object in order to do garbage collection by the method of
@@ -143,13 +148,20 @@ public:
 
 	// info
 	bool info(unsigned inf) const { return bp->info(inf); }
+	bool is_integer() const { return bp->is_integer(); }
+	bool is_real() const { return bp->is_real(); }
+	bool is_positive() const { return bp->is_positive(); }
+	bool is_negative() const { return bp->is_negative(); }
 
 	// operand access
 	size_t nops() const { return bp->nops(); }
+        size_t treesize() const;
 	size_t nsymbols() const;
         bool get_first_symbol(ex &x) const;
         symbolset symbols() const;
         symbolset free_symbols() const;
+        std::unordered_set<unsigned> functions() const;
+        wildset wilds() const;
 	const ex op(size_t i) const { return bp->op(i); }
 	ex sorted_op(size_t i) const;
 	ex operator[](const ex & index) const { return (*bp)[index]; }
@@ -157,6 +169,7 @@ public:
 	ex & let_op(size_t i);
 	ex & operator[](const ex & index);
 	ex & operator[](size_t i);
+        void set_epseq_from(size_t i, ex e) { bp->set_epseq_from(i, e); }
 	ex lhs() const;
 	ex rhs() const;
 
@@ -166,14 +179,18 @@ public:
 	ex imag_part() const { return bp->imag_part(); }
 
 	// pattern matching
-	bool has(const ex & pattern, unsigned options = 0) const { return bp->has(pattern, options); }
+	bool has(const ex & pattern, unsigned options = 0) const
+            { return bp->has(pattern, options); }
 	bool find(const ex & pattern, lst & found) const;
 	bool match(const ex & pattern) const;
-	bool match(const ex & pattern, lst & repl_lst) const { return bp->match(pattern, repl_lst); }
+	bool match(const ex & pattern, lst & repl_lst) const;
+	bool match(const ex & pattern, exmap& map) const
+            { return bp->match(pattern, map); }
+	bool match(const ex & pattern, exvector& vec) const;
+        std::vector<exmap> all_matches(const ex & pattern) const;
 
 	// substitutions
-	ex subs(const exmap & m, unsigned options = 0) const
-                { return bp->subs(m, options); }
+	ex subs(const exmap & m, unsigned options = 0) const;
 	ex subs(const lst & ls, const lst & lr, unsigned options = 0) const;
 	ex subs(const ex & e, unsigned options = 0) const;
 
@@ -199,6 +216,7 @@ public:
 	// expand/collect
 	ex expand(unsigned options=0) const;
 	ex collect(const ex & s, bool distributed = false) const { return bp->collect(s, distributed); }
+        ex collect_powers() const;
 
 	// differentiation and series expansion
 	ex diff(const symbol & s, unsigned nth = 1) const;
@@ -234,6 +252,7 @@ public:
 	numeric max_coefficient() const;
         bool is_linear(const symbol& x, ex& a, ex& b) const;
         bool is_quadratic(const symbol& x, ex& a, ex& b, ex& c) const;
+        bool is_binomial(const symbol& x, ex& a, ex& j, ex& b, ex& n) const;
 
 	// domains
 	void set_domain(unsigned d);
@@ -244,22 +263,16 @@ public:
 	bool is_zero() const;
         bool is_one() const;
         bool is_minus_one() const;
+        bool is_negative_or_minus() const;
+        bool is_num_integer() const;
+        bool is_num_fraction() const;
 	
-	// symmetry
-	ex symmetrize() const;
-	ex symmetrize(const lst & l) const;
-	ex antisymmetrize() const;
-	ex antisymmetrize(const lst & l) const;
-	ex symmetrize_cyclic() const;
-	ex symmetrize_cyclic(const lst & l) const;
-
 	// noncommutativity
 	unsigned return_type() const { return bp->return_type(); }
 	tinfo_t return_type_tinfo() const { return bp->return_type_tinfo(); }
 
 	long gethash() const { return bp->gethash(); }
 
-private:
 	static ptr<basic> construct_from_basic(const basic & other);
 	static basic & construct_from_int(int i);
 	static basic & construct_from_pyobject(PyObject* o);
@@ -277,7 +290,6 @@ private:
 
 // member variables
 
-private:
 	mutable ptr<basic> bp;  ///< pointer to basic object managed by this
 };
 
